function [feasible] = CheckCons(parset,A,b,Aeq,beq,LB,UB,nonlcon,Precision,options)

% CheckCons tests whether a parameterset satisfies the given linear
% inequality and equality, and non-linear constraints
%
% [feasible] = CheckCons(parset,A,b,Aeq,beq,LB,UB,nonlcon,Precision)
%
% INPUTS:
% parset - parameter vector to be tested
% A,b - Matrix A and vector b specifying linear inequality constraints (Ax<b)
% Aeq, beq - Matrix Aeq and vector beq specifying linear equality constraints
% (Ax=b)
% LB - Lower parameter bound vector
% UB - Upper parameter bound vector
% nonlcon - function handle for a non-linear constraint 
% Precision - Vector containing encoding precisions for parameters
% options - BELUGA options structure
%
% OUTPUTS:
% feasible - logical scalar: 1 if all constraints are satisfied, 0 if any
% of the constraints is not satisfied

% note: Due to the encoding/decoding algorithm, children are guaranteed to  
% lie within upper and lower bounds.  LB and UB constraints are not tested 
% in this function. 

feasible = 1; %all constraints satisfied

%determine feasibility of linear inequality constraints
if ~isempty(A) && ~isempty(b)
if any(A*parset'-b > 0)
    feasible = 0; %constraint violation
end
end

%Determine feasibility of linear equality constraints
if ~isempty(Aeq) && ~isempty(beq)
for m=1:length(Aeq(:,1))
    tol = 0;
    %find tolerance of constraint
    for n=1:length(parset)
        tol = tol + abs(Aeq(m,n))*(UB(n)-LB(n))*(10^(-Precision(n)));
    end
    if abs(Aeq(m,:)*parset'-beq(m)) > tol
        feasible = 0; %constraint violation
    end
end
end

% Determine feasibility of non-linear constraints
if ~isempty(nonlcon)
    if feval(nonlcon,parset,LB,UB,Precision) <= options.Tolerance
        feasible = 1;
    else
        feasible = 0;
    end
end