function [Fitness,Population] = FindFitness(ObjectiveFcn,Population,options)

% FindFitness calls the objective function to evaluate the fitness of a given
% population and returns the fitness and evaluated population in descending 
% order of fitness rank

% [Fitness,Population] = FindFitness(ObjectiveFcn,Population,options)
%
% INPUTS
% ObjectiveFcn - Function handle of the objective function
% Population - Population whose fitness is to be evaluated
% options - BELUGA options structure
% 


%% Parallelized

if strcmp(options.UseParallel,'distcomp')
    
    All_Managers = findResource('jobmanager','lookupURL','zeke.phys.mcw.edu');
    %get(All_Managers);  % display a data structure of obtained managers
    Job_Manager=All_Managers(1); % choose a job manager to use
    
    for j = 1:floor(size(Population,1)/options.NumberOfNodes)
    
        Job1=createJob(Job_Manager);

        set(Job1,'MaximumNumberOfWorkers',options.NumberOfWorkers,...
            'FileDependencies',options.FileDependencyList);
        Job1_ID = Job1.ID;
    
        for k = 1:options.NumberOfNodes
            task(k) = createTask(Job1,ObjectiveFcn,1,...
                {Population(((j-1)*options.NumberOfNodes+k),:)}); %#ok<AGROW>
        end

        submit(Job1);
        waitForState(Job1,'finished');
        output = getAllOutputArguments(Job1);
        SSE(((j-1)*options.NumberOfNodes+1):...
            ((j-1)*options.NumberOfNodes+options.NumberOfNodes),1) = output{:};

        Job2Kill = findJob(Job_Manager,'UserName',options.UserName,'ID',Job1_ID);
        destroy(Job2Kill);
        clear Job1_ID Job1 task Job2Kill output
    end
    
    if ((size(Population,1)/options.NumberOfNodes) - ...
            floor(size(Population,1)/options.NumberOfNodes)) > 0
        j = j+1;
        
        Job1=createJob(Job_Manager);

        set(Job1,'MaximumNumberOfWorkers',options.NumberOfWorkers,...
            'FileDependencies',options.FileDependencyList);
        Job1_ID = Job1.ID;
        
        for k = 1:(((size(Population,1)/options.NumberOfNodes) - ...
                floor(size(Population,1)/options.NumberOfNodes))*options.NumberOfNodes)
            task(k) = createTask(Job1,ObjectiveFcn,1,...
                {Population(((j-1)*options.NumberOfNodes+k),:)}); %#ok<AGROW>
        end
        
        submit(Job1);
        waitForState(Job1,'finished');
        output = getAllOutputArguments(Job1);
        SSE(((j-1)*options.NumberOfNodes+1):...
            ((j-1)*options.NumberOfNodes+k),1) = output{:};

        Job2Kill = findJob(Job_Manager,'UserName',options.UserName,'ID',Job1_ID);
        destroy(Job2Kill);
        clear Job1_ID Job1 task Job2Kill output
    end
end

%% Using parfor to run on a local multicore machine
%% Open matlabpool for 'multicore' option setting in options.UseParallel
if strcmp(options.UseParallel,'matlabpool')
    SSE = zeros(size(Population,1),1); % preallocate for speed
   
    parfor i = 1:size(Population,1)
        SSE(i) = feval(ObjectiveFcn,Population(i,:));
    end
end

%% Local evaluation on a single core
if strcmp(options.UseParallel,'off')
    SSE = zeros(size(Population,1),1); % preallocate for speed
    for i = 1:size(Population,1)
        SSE(i) = feval(ObjectiveFcn,Population(i,:));
    end   
end 

%% Return Fitness vector and evaluated Population ranked in descending order of fitness 

Fitness = SSE;

