function [x,fval,output,finalpop,fitness] = beluga(varargin)
% BELUGA (Beard Lab GA based Optimizer, Implementation of Paul Charbonneau's PIKAIA)
% Written by: Scott Bugenhagen and Kalyan Vinnakota
% Last Modified: 07/24/12
%
% [x,fval,exitflag,output,finalpop,fitness] = beluga(ObjectiveFcn,nvars,A,b,Aeq,beq,LB,UB,nonlcon,options)
% 
% INPUTS:
% ObjectiveFcn - Function handle of the objective/cost function (mandatory input)
% nvars - number of adjustable parameters (mandatory input) 
% A,b - Matrix A and vector b specifying linear inequality constraints
% (Ax<b), use empty matrices if there are no linear inequality constraints
% Aeq, beq - Matrix Aeq and vector beq specifying linear equality constraints
% (Ax=b), use empty matrices if there are no linear equality constraints
% LB - Lower parameter bound vector, (mandatory finite double vector input)
% UB - Upper parameter bound vector, (mandatory finite double vector input)
% nonlcon - function handle for a non linear constraint, use an empty
% matrix if there are no nonlinear constraints (currently not implemented, use [])
% options - BELUGA options structure, default options will be used if no
% options are specified. See belugaoptset.m for information on this input 
%
% OUTPUTS:
% x - optimized parameter set i.e, best individual of the final population
% fval - value of costfunction for the optimized parameter set 
% output - 6 column Matrix containing information on the progress of optimization. 
% Columns contain 1)Generation 2)Fitness of best individual 3)Fitness of median individual
% 4) Distance measure 5) Probability of mutation 6) Selection pressure
% finalop - Final population
% fitness - Fitness vector of of the final population
%
% This code is based on the PIKAIA 1.2 GA by Paul Charbonneau
% References and links to fulltext:
% 1. Charbonneau, P., and Knapp, B. 1995, A User's guide to PIKAIA 1.0, NCAR
% Technical Note 418+IA (Boulder: National Center for Atmospheric Research)
% URL:  http://www.hao.ucar.edu/Public/models/pikaia/ugtc.ps
% 2. Charbonneau, P., 2002, Release Notes for PIKAIA 1.2, NCAR Technical Note
% 451+STR (Boulder: National Center for Atmospheric Research).
% URL: http://www.hao.ucar.edu/Public/models/pikaia/relnotes.ps
% 3. Charbonneau, P., 2002, An Introduction to Genetic Algorithms for
% Numerical Optimization, NCAR Technical Note 450+IA
%(Boulder: National Center for Atmospheric Research).
% URL: http://www.hao.ucar.edu/Public/models/pikaia/tutorial/tutorial.ps
%


%% load inputs

inputs(1:10) = {[]};

for i = 1:length(varargin)
    inputs(i) = varargin(i);
end

ObjectiveFcn = inputs{1};
nvars        = inputs{2};
A            = inputs{3};
b            = inputs{4};
Aeq          = inputs{5};
beq          = inputs{6};
LB           = inputs{7};
UB           = inputs{8};
nonlcon      = inputs{9};
options      = inputs{10};

% Load default options of no options stucture is supplied
if isempty(options)
    options = belugaoptset;
end

%% Validate Inputs
% Validate bounds

if isempty(LB) || isempty(UB)
    error('LB and UB must be specified')
end
if (length(LB)~=nvars || length(UB)~=nvars)
    error('Lengths of LB and UB must match nvars')
end
if any(~isfinite(LB) + ~isfinite(UB))
    error('Elements of LB and UB cannot be Inf or NaN')
end
if any(LB>UB)
    error('Elements of LB must be greater than those of UB')
end

% Validate Precision vector

if isscalar(options.Precision)
    options.Precision = options.Precision*ones(1,nvars);
elseif length(options.Precision)~=nvars
    error('Error: length of Precision vector must be equal to nvars')
end


%% Initialization
% change state of rand before start
rand('twister',sum(100*clock)); %#ok<RAND>

GenerationCount = 1; %first generation

% pmutcrit = FindCriticalMutRate(sum(options.Precision),options.PopulationSize); %critical mutation rate
pmutmx = 0.25; %max mutation rate
pmutmn = 0.00025; %min mutation rate

%mutation rate
if strcmp(options.MutationAdapt,'on')
    pm = pmutmx;
elseif strcmp(options.MutationAdapt,'off')
    pm = pmutcrit*(1 - 0.001);
end

SPmax = 1;
SPmin = 0.001;

%selection pressure
if strcmp(options.SelectPressAdapt,'on')
    SP = 1;
elseif strcmp(options.SelectPressAdapt,'off')
    SP = options.SelectionPressure;
end


%% First Generation

% Generate Initial Population
Population = feval(options.CreationFcn,A,b,Aeq,beq,LB,UB,nonlcon,options);

% Seed Population if Given InitialPop
if ~isempty(options.InitPop)
        Population = options.InitPop;
end

if strcmp(options.UseParallel,'matlabpool')
     eval(['matlabpool open ',options.MatlabpoolConfig]) 
end

% Evaluate Fitness of Individuals in Population
Fitness = FindFitness(ObjectiveFcn,Population,options);

% Measure Diversity (Clustering) of Population
Distance = feval(options.DistanceFcn,Fitness);

% Generate progress information
UserOutput  = zeros(options.Generations,6);
UserOutput(GenerationCount,:) = [GenerationCount,min(Fitness),...
    Fitness(length(Fitness)*RankFitness(Fitness)==round(length(Fitness)/2)),...
    Distance,pm,SP];

% Adapt Mutation Rate
if strcmp(options.MutationAdapt,'on')
    pm = AdaptMutation(Distance,pm,pmutmx,pmutmn,options.AdaptRate);
end

% Adapt Selection Pressure
if strcmp(options.SelectPressAdapt,'on')
    SP = AdaptSelectPressure(Distance,SP,SPmax,SPmin,options.AdaptRate);
end

%% GA Main

% Generational cycle
for n = 2:options.Generations
    GenerationCount= n;
    display(['Best Fitness: ',num2str(min(Fitness))])
    display(['Generation: ',num2str(n)])
    % Produce the next generation
    newPop = feval(options.Reproduction,Fitness,Population,...
        SP,pm,nvars,A,b,Aeq,beq,LB,UB,nonlcon,options);
    
    % Evaluate Fitness of Individuals in Population
    newFitness = FindFitness(ObjectiveFcn,newPop,options);

    % Enforce elitism if chosen
    if strcmp(options.Elitism,'on')
        elite_prevGen = Population(find(Fitness==(min(Fitness)),1),:);
        
        % replace a random less fit (greater SSE) member of the present 
        % generation with this elite individual from the previous
        % generation
        if any(newFitness>min(Fitness))
            lessfitinds = find(newFitness>min(Fitness));
            randind = lessfitinds(floor(length(lessfitinds)*rand)+1);
            newPop(randind,:) = elite_prevGen;
            newFitness(randind)= min(Fitness);
        end
           Population = newPop; 
           Fitness = newFitness; 
    end
    
    % Measure Diversity (Clustering) of Population
    Distance = feval(options.DistanceFcn,Fitness);
    
    % Generate progress information
    UserOutput(GenerationCount,:) = [GenerationCount,min(Fitness),...
    Fitness(length(Fitness)*RankFitness(Fitness)==round(length(Fitness)/2)),...
    Distance,pm,SP];

    % Adapt Mutation Rate
    if strcmp(options.MutationAdapt,'on')
        pm = AdaptMutation(Distance,pm,pmutmx,pmutmn,options.AdaptRate);
    end

    % Adapt Selection Pressure
    if strcmp(options.SelectPressAdapt,'on')
        SP = AdaptSelectPressure(Distance,SP,SPmax,SPmin,options.AdaptRate);
    end 
    
end
if strcmp(options.UseParallel,'matlabpool')
    matlabpool close
end
%% Return
x = Population(find(Fitness==(min(Fitness)),1),:); %Fittest individual from final population
fval = Fitness(find(Fitness==min(Fitness),1));
output = UserOutput;
finalpop = Population; 
fitness = Fitness;


