function [options] = belugaoptset(varargin)
%
% BELUGAOPTSET Alters BELUGA options structure from default settings
%
%   OPTIONS = BELUGAOPTS() returns the options structure with default
%   settings
%
%   OPTIONS = BELUGAOPTS('PARAM',VALUE) to alter a single parameter setting
%
%   OPTIONS = BELUGAOPTS('PARAM1',VALUE1,'PARAM2','VALUE2'......) to alter
%   multiple parameter settings
%
% BELUGAOPTSET PARAMETERS
%     PopulationSize      - Positive scalar indicating the number of
%                           individuals
%                         [ positive scalar | {50} ]
%     Generations         - Maximum number of generations allowed
%                         [ positive scalar | {100} ]
%     Precision           - Number of significant digits for decimal
%                           encoding
%                         [ positive integer vector or scalar | {4} ]
%     XoverRate           - Crossover rate
%                         [ positive scalar | {0.85} ]
%     Elitism             - Option to pass on the elite individual(s) to
%                           the next generation without any change
%                         [ 'off' | {'on'} ]
%     MutationAdapt       - Option to switch on adaptive mutation
%                         [ 'off' | {'on'} ]
%     SelectPressAdapt    - Option to switch on adaptive selection pressure
%                         [ {'off'} | 'on' ]
%     CreationFcn         - Function used to create initial population
%                         [ function handle | {@CreateUniform} ]
%     EncodeFcn           - Function used to encode parameter values into
%                           chromosomes
%                         [ function handle | {@EncodeDecimal} ]
%     DecodeFcn           - Function used to decode parameter values from
%                           chromosomes
%                         [ function handle | {@DecodeDecimal} ]
%     XoverFcn            - Function used to generate offspring from
%                           crossingover parent chromosomes
%                         [ function handle | {@XoverHybrid} ]
%     MutateFcn           - Function used in mutating genes
%                         [ {'@UniformMutation'} ]
%     FitnessScalingFcn   - Function used to scale fitness scores.
%                         [ function handle | {@RankFit} ]
%     SelectionFcn        - Function for selecting parents
%                         [ function handle | {@Roulette} ]
%     UseParallel         - Set options for distributed/parallel
%                           computation
%                         [ {'off'} | 'distcomp' | 'matlabpool' ]
%     MatlabpoolConfig    - Set options for 'multicore' setting in
%                           distributed/parallel computation
%                         [ string | {'local'} ]
%     NumberOfNodes       - Maximum number of nodes to be used for
%                           distributed computing
%                         [ positive scalar | {60} ]
%     NumberOfWorkers     -
%                         [ positive scalar | {60} ]
%     FileDependencyList  - File dependency list in a cell array for
%                           distributed computing
%                         [ cell array of strings | {{}} ]
%     UserName            - Username for your distributed computing
%                           cluster
%                         [ string | {''} ]
%     PlotFcn           - Function for displaying the progress of the
%                           optimization
%                         [ function handle | @ConvergeInfo | {[]} ]
%     SelectionPressure   -
%                         [ {[0;1]} ]
%     DistanceFcn         - Function for calculating the average distance
%                         between members of a population (?)
%                         [ function handle | {@FitnessBased} | @DistanceBased ]
%     MutationCreep       - Option to switch on mutation creep
%                         [ 'off' | {'on'}]
%     AdaptRate           - Rate of adaptation of mutation rate
%                         [ positive scalar | {1.5} ]
%     Tolerance           - Tolerance for satisfying constraints
%                         [ positive scalar | {1e-4} ]
%     InitPop             - Initial population for seeding the GA
%                         [ double vector | {[]} ]
%     Reproduction        - Reproduction plan for replacing generations
%                         [ function handle | {@genrep} ]
%     
%
%% Default assignments to the options structure
options.PopulationSize = 50;
options.Generations = 100;
options.Precision = 4;
options.XoverRate = 0.85;
options.Elitism = 'on';
options.MutationAdapt = 'on';
options.SelectPressAdapt = 'off';
options.CreationFcn = @CreateUniform;
options.EncodeFcn = @EncodeDecimal;
options.DecodeFcn = @DecodeDecimal;
options.XoverFcn = @XoverHybrid;
options.MutateFcn = @UniformMutation;
options.FitnessScalingFcn = @RankFitness;
options.SelectionFcn = @Roulette;
options.UseParallel = 'off';
options.NumberOfNodes = 60;
options.NumberOfWorkers = 60;
options.FileDependencyList = {};
options.UserName = [];
options.PlotFcn = [];
options.SelectionPressure = 1; 
options.DistanceFcn = @FitnessBased;
options.MutationCreep = 'on';
options.AdaptRate = 1.5;
options.Tolerance = 1E-4;
options.InitPop = [];
options.Reproduction = @genrep;
options.MatlabpoolConfig = 'local';
optionsfieldnames = fieldnames(options);
%% Assign specified options
if ~isempty(varargin)
    if (mod(length(varargin),2)==0)
        for i=1:length(varargin)/2
            if(~isempty(strmatch(varargin{2*i-1},optionsfieldnames))) % check for existence of fieldname
                if(~isempty(varargin{2*i})) % do not set field if specified value is []
                    options=setfield(options,varargin{2*i-1},varargin{2*i}); %#ok<SFLD>
                end
            else
                error([varargin{2*i-1},' - field does not exist'])
            end
        end
    else
        error('Number of options set do not match the number of values specified')
    end
end

%% Validate Inputs
% Validate Precision vector
if any((options.Precision-floor(options.Precision))>0) || any(options.Precision <= 0)
    error('Elements of Precision must be positive integers')
end

% Validate cluster settings for distributed computing

if strcmp(options.UseParallel,'distcomp')    
    if isempty(options.UserName)
        error('UserName must be specified for distributed computing')
    end
    
    if options.PopulationSize < options.NumberOfNodes
        error('Error: PopulationSize must be greater or equal to NumberOfNodes');
    end

    if ((options.PopulationSize/options.NumberOfNodes) - ...
            floor(options.PopulationSize/options.NumberOfNodes)) > 0
        disp('Warning: PopulationSize should be an integer multiple of NumberOfNodes for maximum efficiency');
    end

    if isempty(options.NumberOfWorkers)
        options.NumberOfWorkers = options.NumberOfNodes;
    end
end

