function [nextGen] = genrep(Fitness,Population,...
   SP,pm,nvars,A,b,Aeq,beq,LB,UB,nonlcon,options)

% genrep computes a new generation to replace the previous generation. This 
% function is used in a reproduction plan for total replacement of a prior 
% generation
%
% [nextGen] = genrep(Fitness,Population,...
%    SP,pm,nvars,A,b,Aeq,beq,LB,UB,nonlcon,options)
%
% INPUTS:
% Fitness - vector of sum of squares of residuals (SSE)
% Population - Parameter population matrix
% SP - Selection pressure (fitness differential)
% pm - Probability of mutation
% nvars - number of adjustable parameters 
% A,b - Matrix A and vector b specifying linear inequality constraints
% (Ax<b), use empty matrices if there are no linear inequality constraints
% Aeq, beq - Matrix Aeq and vector beq specifying linear equality constraints
% (Ax=b), use empty matrices if there are no linear equality constraints
% LB - Lower parameter bound vector, (mandatory finite double vector input)
% UB - Upper parameter bound vector, (mandatory finite double vector input)
% nonlcon - function handle for a non linear constraint, use an empty
% matrix if there are no nonlinear constraints (currently not implemented, use [])
% options - BELUGA options structure, default options will be used if no
% options are specified. See belugaoptset.m for information on this input
%
% OUTPUTS:
% nextGen - Parameter population matrix of the next generation
%

ScaledFitness = feval(options.FitnessScalingFcn,Fitness);
oldPop = Population;
% encode oldPop

oldGenome = feval(options.EncodeFcn,oldPop,LB,UB,options.Precision);

nextGen_Genome = zeros(options.PopulationSize,sum(options.Precision));% preallocate for speed
for i = 1:floor(options.PopulationSize/2)
    % Selection
    P1 = feval(options.SelectionFcn,ScaledFitness,SP); % select parent1
    P1eqP2 = 1; % set equality condition to intiate the while loop below
    while P1eqP2 % run the loop while parent2=parent1 i.e. quit when parent 2 ~= parent1
        P2 = feval(options.SelectionFcn,ScaledFitness,SP); % select parent2
        P1eqP2 = (P1==P2); % evaluate equality condition
    end
    
    % Crosssover operation
    [offspring1, offspring2] = feval(options.XoverFcn,oldGenome(P1,:),oldGenome(P2,:),options.XoverRate);
    % Mutate offspring1
    offspring1 = feval(options.MutateFcn,pm,...
        options.MutationCreep,options.Precision,offspring1);
    % Mutate offspring2
    offspring2 = feval(options.MutateFcn,pm,...
        options.MutationCreep,options.Precision,offspring2);
    
    % Generate new population by generational replacement
    % Generational replacement w/0 elitism. When chosen, elitism is enforced in the
    % main program
    
    if 2*i-1 <= options.PopulationSize
    nextGen_Genome(2*i-1,:) = offspring1;
    end
    if 2*i <= options.PopulationSize
    nextGen_Genome(2*i,:) = offspring2;
    end
    
end

nextGen = feval(options.DecodeFcn,nextGen_Genome,LB,UB,options.Precision);